# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawGenerationsClient
from ..common.types.model import Model
from ..common.types.input import Input
from ..common.types.generation_options import GenerationOptions
from ..core.request_options import RequestOptions
from ..common.types.generation import Generation
from ..common.types.generation_id import GenerationId
from ..common.types.generation_status import GenerationStatus
from ..common.types.estimated_generation_cost import EstimatedGenerationCost
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawGenerationsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class GenerationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGenerationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawGenerationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGenerationsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        model: Model,
        input: typing.Sequence[Input],
        options: typing.Optional[GenerationOptions] = OMIT,
        webhook_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Generation:
        """
        Parameters
        ----------
        model : Model
            name of the model to use for generation.

        input : typing.Sequence[Input]
            Array of input objects. Must include one video input item and one audio input item. Audio input items can be provided as either: recorded/captured audio url or a text-to-speech input with tts provider configuration.

        options : typing.Optional[GenerationOptions]
            additional options available for generation.

        webhook_url : typing.Optional[str]
            webhook url for generation status updates. once the generation completes we will send a POST request to the webhook url with the generation data.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Generation
            Job created successfully

        Examples
        --------
        from sync import Sync
        from sync.common import Audio, GenerationOptions, Video

        client = Sync(
            api_key="YOUR_API_KEY",
        )
        client.generations.create(
            input=[
                Video(
                    url="https://assets.sync.so/docs/example-video.mp4",
                ),
                Audio(
                    url="https://assets.sync.so/docs/example-audio.wav",
                ),
            ],
            model="lipsync-2",
            options=GenerationOptions(
                sync_mode="loop",
            ),
        )
        """
        response = self._raw_client.create(
            model=model, input=input, options=options, webhook_url=webhook_url, request_options=request_options
        )
        return response.data

    def get(self, id: GenerationId, *, request_options: typing.Optional[RequestOptions] = None) -> Generation:
        """
        Parameters
        ----------
        id : GenerationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Generation
            Job status retrieved successfully

        Examples
        --------
        from sync import Sync

        client = Sync(
            api_key="YOUR_API_KEY",
        )
        client.generations.get(
            id="6533643b-aceb-4c40-967e-d9ba9baac39e",
        )
        """
        response = self._raw_client.get(id, request_options=request_options)
        return response.data

    def list(
        self,
        *,
        status: typing.Optional[GenerationStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Generation]:
        """
        Parameters
        ----------
        status : typing.Optional[GenerationStatus]
            Filter generations by status

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Generation]
            Generations retrieved successfully

        Examples
        --------
        from sync import Sync

        client = Sync(
            api_key="YOUR_API_KEY",
        )
        client.generations.list()
        """
        response = self._raw_client.list(status=status, request_options=request_options)
        return response.data

    def estimate_cost(
        self,
        *,
        model: Model,
        input: typing.Sequence[Input],
        options: typing.Optional[GenerationOptions] = OMIT,
        webhook_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[EstimatedGenerationCost]:
        """
        Parameters
        ----------
        model : Model
            name of the model to use for generation.

        input : typing.Sequence[Input]
            Array of input objects. Must include one video input item and one audio input item. Audio input items can be provided as either: recorded/captured audio url or a text-to-speech input with tts provider configuration.

        options : typing.Optional[GenerationOptions]
            additional options available for generation.

        webhook_url : typing.Optional[str]
            webhook url for generation status updates. once the generation completes we will send a POST request to the webhook url with the generation data.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[EstimatedGenerationCost]
            Generations retrieved successfully

        Examples
        --------
        from sync import Sync
        from sync.common import Audio, GenerationOptions, Video

        client = Sync(
            api_key="YOUR_API_KEY",
        )
        client.generations.estimate_cost(
            input=[
                Video(
                    url="https://assets.sync.so/docs/example-video.mp4",
                ),
                Audio(
                    url="https://assets.sync.so/docs/example-audio.wav",
                ),
            ],
            model="lipsync-2",
            options=GenerationOptions(
                sync_mode="loop",
            ),
        )
        """
        response = self._raw_client.estimate_cost(
            model=model, input=input, options=options, webhook_url=webhook_url, request_options=request_options
        )
        return response.data


class AsyncGenerationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGenerationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawGenerationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGenerationsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        model: Model,
        input: typing.Sequence[Input],
        options: typing.Optional[GenerationOptions] = OMIT,
        webhook_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Generation:
        """
        Parameters
        ----------
        model : Model
            name of the model to use for generation.

        input : typing.Sequence[Input]
            Array of input objects. Must include one video input item and one audio input item. Audio input items can be provided as either: recorded/captured audio url or a text-to-speech input with tts provider configuration.

        options : typing.Optional[GenerationOptions]
            additional options available for generation.

        webhook_url : typing.Optional[str]
            webhook url for generation status updates. once the generation completes we will send a POST request to the webhook url with the generation data.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Generation
            Job created successfully

        Examples
        --------
        import asyncio

        from sync import AsyncSync
        from sync.common import Audio, GenerationOptions, Video

        client = AsyncSync(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.generations.create(
                input=[
                    Video(
                        url="https://assets.sync.so/docs/example-video.mp4",
                    ),
                    Audio(
                        url="https://assets.sync.so/docs/example-audio.wav",
                    ),
                ],
                model="lipsync-2",
                options=GenerationOptions(
                    sync_mode="loop",
                ),
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            model=model, input=input, options=options, webhook_url=webhook_url, request_options=request_options
        )
        return response.data

    async def get(self, id: GenerationId, *, request_options: typing.Optional[RequestOptions] = None) -> Generation:
        """
        Parameters
        ----------
        id : GenerationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Generation
            Job status retrieved successfully

        Examples
        --------
        import asyncio

        from sync import AsyncSync

        client = AsyncSync(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.generations.get(
                id="6533643b-aceb-4c40-967e-d9ba9baac39e",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(id, request_options=request_options)
        return response.data

    async def list(
        self,
        *,
        status: typing.Optional[GenerationStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Generation]:
        """
        Parameters
        ----------
        status : typing.Optional[GenerationStatus]
            Filter generations by status

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Generation]
            Generations retrieved successfully

        Examples
        --------
        import asyncio

        from sync import AsyncSync

        client = AsyncSync(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.generations.list()


        asyncio.run(main())
        """
        response = await self._raw_client.list(status=status, request_options=request_options)
        return response.data

    async def estimate_cost(
        self,
        *,
        model: Model,
        input: typing.Sequence[Input],
        options: typing.Optional[GenerationOptions] = OMIT,
        webhook_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[EstimatedGenerationCost]:
        """
        Parameters
        ----------
        model : Model
            name of the model to use for generation.

        input : typing.Sequence[Input]
            Array of input objects. Must include one video input item and one audio input item. Audio input items can be provided as either: recorded/captured audio url or a text-to-speech input with tts provider configuration.

        options : typing.Optional[GenerationOptions]
            additional options available for generation.

        webhook_url : typing.Optional[str]
            webhook url for generation status updates. once the generation completes we will send a POST request to the webhook url with the generation data.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[EstimatedGenerationCost]
            Generations retrieved successfully

        Examples
        --------
        import asyncio

        from sync import AsyncSync
        from sync.common import Audio, GenerationOptions, Video

        client = AsyncSync(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.generations.estimate_cost(
                input=[
                    Video(
                        url="https://assets.sync.so/docs/example-video.mp4",
                    ),
                    Audio(
                        url="https://assets.sync.so/docs/example-audio.wav",
                    ),
                ],
                model="lipsync-2",
                options=GenerationOptions(
                    sync_mode="loop",
                ),
            )


        asyncio.run(main())
        """
        response = await self._raw_client.estimate_cost(
            model=model, input=input, options=options, webhook_url=webhook_url, request_options=request_options
        )
        return response.data
